import java.util.Random;

/**
 * Simulátor hry kostky. Hody kostkami simuluje pomocí
 * generátoru náhodných čísel.
 */
public class Craps {
	
	// generátor náhodných čísel
	private Random generator = new Random();
	
	// výčtový typ s konstantami reprezentujícími stav hry
	private enum Status { CONTINUE, WON, LOST };
	
	// konstanty reprezntující některé hodnoty
	// součtu hozeného na dvou kostkách
	private final static int SNAKE_EYES = 2;
	private final static int TREY = 3;
	private final static int SEVEN = 7;
	private final static int YO_LEVEN = 11;
	private final static int BOX_CARS = 12;
	
	// frekvence výher na určitý počet hodů
	// [0] nepoužito
	// [1] počet výher na první hod
	// [2] počet výher na druhý hodech
	// ...
	// [20] počet výher na dvacátý hod
	// [21] počet výher na více neľ dvace hodů
	private int[] winFreq = new int[22];
	private int[] lostFreq = new int[22];
    
    /**
     * Celkový počet hodů kostkou ze všech her.
     * */
    private int rollCount;
	
	// počet her, které byly hrány
	private int gameCount;
	
	
	/**
	 * Provede simulaci jedné hry kostky.
	 */
	public void play() {
		// počet bodů pokud hráč nevyhraje na první hod
		int myPoint = 0;
		// stav hry
		Status gameStatus;
		// první hod kostkami
		int sumOfDice = rollDice();
		
		// určení stavu na základě prvního hodu
		switch (sumOfDice) {
		case SEVEN:
		case YO_LEVEN:
			gameStatus = Status.WON;
			winFreq[1]++;
			break;
		case SNAKE_EYES:
		case TREY:
		case BOX_CARS:
			gameStatus = Status.LOST;
			lostFreq[1]++;
			break;
		default:
			gameStatus = Status.CONTINUE;
			myPoint = sumOfDice;
			/* Tento tisk pro simulaci nepotřebujeme
			System.out.printf("Hráč čeká na: %d\n", myPoint);
			*/
			break;
		}
		
		int pocetHodu = 1;
		// dokud není hra u konce
		while (gameStatus == Status.CONTINUE) {
			// další hod kostkami
			sumOfDice = rollDice();
			pocetHodu++;

			// určení stavu
			if (sumOfDice == myPoint) {
				gameStatus = Status.WON;
				if (pocetHodu > 20) {
					winFreq[21]++;
				} else {
					winFreq[pocetHodu]++;
				}
			} else if (sumOfDice == SEVEN) { 
				gameStatus = Status.LOST;
				if (pocetHodu > 20) {
					lostFreq[21]++;
				} else {
					lostFreq[pocetHodu]++;
				}
			}
		}
		
		// zobrazení zprávy o výsledku hry
		/* Tento tisk pro simulaci nepotřebujeme
		if (gameStatus == Status.WON) {
			System.out.println("Hráč vyhrál.");
		} else {
			System.out.println("Hráč prohrál.");
		}
		*/
		
		gameCount++;
	}

	
	/**
	 * Provede jeden hod dvěma kostkami a vrátí součet
	 * hodnot, které padly. Informace o hodu zároveň
	 * vytiskne na standardní výstup.
	 * 
	 * @return		součet hodnot na obou kostkách.
	 */
	public int rollDice() {
        rollCount++;
		// výběř náhodných hodnot
		int die1 = 1 + generator.nextInt(6);
		int die2 = 1 + generator.nextInt(6);
		
		// součet hodnot obou kostek
		int sum = die1 + die2;
		
		// zobrazení výsledku hodu
		/* Tento tisk pro simulaci nepotřebujeme.
		System.out.printf("Hráč hodil %d + %d = %d\n", 
				die1, die2, sum);
		*/
		
		// vrácení součtu
		return sum;
	}
	
	/**
	 * Vrátí součet všech prvků v poli array.
	 * 
	 * @param array		pole, jehoľ prvky se budou sčítat.
	 * @return			součet prvků v poli array.
	 */
	private int sumArray(int[] array) {
		int total = 0;
		
		for (int item: array) {
			total += item;
		}
		
		return total;
	}

	/**
	 * Vrátí součet všech prvků v poli array.
	 * 
	 * @param array		pole, jehož prvky se budou sčítat.
     * @param zacatek   index od kterého budu sčítat
	 * @return			součet prvků v poli array.
	 */
	private int sumArray(int[] array, int zacatek) {
		int total = 0;
		
		for (int i = zacatek; i < array.length; i++) {
			total += array[i];
		}
		
		return total;
	}
	
	/**
	 * Vrátí hodnotu maximálního prvku v poli array.
	 * 
	 * @param array		pole, ve kterém se hledá maximální hodnota.
	 * @return			hodnota maximálního prvku v poli array.
	 */
	private int maxArray(int[] array) {
		int max = array[0];
		
		for (int item: array) {
			if (item > max) {
				max = item;
			}
		}
		
		return max;
	}

	/**
	 * Vrátí hodnotu maximálního prvku v poli array.
	 * 
	 * @param array		pole, ve kterém se hledá maximální hodnota.
	 * @return			hodnota maximálního prvku v poli array.
	 */
	private double maxArray(double[] array) {
		double max = array[0];
		
		for (double item: array) {
			if (item > max) {
				max = item;
			}
		}
		
		return max;
	}
	
	/**
	 * Vytiskne sloupcový graf na základě pole array.
	 * Pozor: metoda ignoruje prvek s indexem 0!
	 * Pozor: metoda se chová s posledním prvekm jinak!
	 * 
	 * @param array		pole, podle kterého se vytiskne graf.
	 */
	private void printBarGraph(int[] array) {
		// počet hvězdiček reprezentující maximální hodnotu
		// final = je to konstanta
		final int MAX_GRAPH_WIDTH = 60;
		
		// maximální hodnota v poli, které bude odpovídat
		// maximální počet hvězdiček MAX_GRAPH_WIDTH
		int maxValue = maxArray(array);
		
		for (int i = 1; i < array.length; i++) {
			if (i == array.length - 1) {
				System.out.printf(">%2d: %5d ", i, array[i]);
			} else {
				System.out.printf("%3d: %5d ", i, array[i]);
			}
			
			int starCount = array[i] * MAX_GRAPH_WIDTH / maxValue;
			for (int star = 0; star < starCount; star++) {
				System.out.print("*");
			}
			System.out.println();
		}
	}

	/**
	 * Vytiskne sloupcový graf na základě pole array.
	 * Pozor: metoda ignoruje prvek s indexem 0!
	 * 
	 * @param array		pole, podle kterého se vytiskne graf.
	 */
	private void printBarGraph(double[] array) {
		// počet hvězdiček reprezentující maximální hodnotu
		// final = je to konstanta
		final int MAX_GRAPH_WIDTH = 60;
		
		// maximální hodnota v poli, které bude odpovídat
		// maximální počet hvězdiček MAX_GRAPH_WIDTH
		double maxValue = maxArray(array);
		
		for (int i = 1; i < array.length; i++) {
			System.out.printf("%3d: %.5f ", i, array[i]);
			
			int starCount =(int) ((array[i] / maxValue) * (double) MAX_GRAPH_WIDTH);
			for (int star = 0; star < starCount; star++) {
				System.out.print("*");
			}
			System.out.println();
		}
	}
	
	/**
	 * Vrátí podíl vyhraných her vzhledem k celkovému počtu odehraných her.
	 * 
	 * @return		podíl vyhraných her.
	 */
	public double getWinChance() {
		return (double) sumArray(winFreq) / gameCount;
	}

	/**
	 * Vrátí podíl prograných her vzhledem k celkovému počtu odehraných her.
	 * 
	 * @return		podíl vyhraných her.
	 */
	public double getLostChance() {
		return (double) sumArray(lostFreq) / gameCount;
	}
	
	/**
	 * Vrátí celkový počet odehraných her.
	 * 
	 * @return		počet odehraných her.
	 */
	public int getGameCount() {
		return gameCount;
	}
	
	/**
	 * Vytiskne testovací sloupcový graf.
	 */
	public void testBarGraph() {
		int[] pole = {0, 1, 2, 3, 4, 5, 6, 7};
		printBarGraph(pole);
	}

    /**
     * Vytiskne sloupcový graf frekvence výher na daný počet hodů.
     * */
    public void prehledVyher()
    {
        printBarGraph(winFreq);
    }
    /**
     * Vytiskne sloupcový graf frekvence proher na daný počet hodů.
     * */
    public void prehledProher()
    {
        printBarGraph(lostFreq);
    }

    /**
     *  Vraci prumernou delku hry
     *
     *  @return Prumerna delka hry
     **/
    public double prumernaDelkaHry()
    {
        return (double) rollCount/gameCount;
    }

    /**
     * Vypocet a zobrazeni pravdepodobnosti vyhry v zavislosti na
     * poctu hodu.
     * */

    public void prehledVyherDleDelkyHry()
    {
        double[] p = new double[21];

        for(int i = 1; i < p.length; i++)
        {
            double sum = 
               (double) (sumArray(winFreq,i+1)+sumArray(lostFreq,i+1));
            if(sum !=0)
                p[i] = (double)sumArray(winFreq,i+1)/ sum;
        }

        printBarGraph(p);
    }
}
